@@ -6,18 +6,19 @@ import logging
 import re
 import urllib.parse
 from contextlib import suppress
-from typing import Tuple, Dict, Optional, List, TYPE_CHECKING, IO, Sequence, Union, Pattern
+from typing import Tuple, Dict, Optional, List, TYPE_CHECKING, IO, Union, Pattern
 
-import telegram
+import telegram  # lgtm [py/import-and-import-from]
 from PIL import Image
-from telegram import Update, Message, Chat, TelegramError
+from telegram import Update, Message, Chat, TelegramError, InlineKeyboardButton, ChatAction, InlineKeyboardMarkup, \
+    ParseMode
 from telegram.error import BadRequest
 from telegram.ext import ConversationHandler, CommandHandler, CallbackQueryHandler, CallbackContext, Filters, \
     MessageHandler
 
 from ehforwarderbot import coordinator, Chat, Channel, MsgType
 from ehforwarderbot.channel import SlaveChannel
-from ehforwarderbot.chat import GroupChat, SystemChatMember
+from ehforwarderbot.chat import SystemChatMember
 from ehforwarderbot.exceptions import EFBChatNotFound, EFBOperationNotSupported
 from ehforwarderbot.types import ModuleID, ChatID, MessageID
 from . import utils
@@ -52,7 +53,7 @@ class ChatListStorage:
         self.channels: Dict[ModuleID, SlaveChannel] = dict()
         self.chats = chats.copy()  # initialize chats with setter.
         self.offset: int = offset
-        self.update: Optional[telegram.Update] = None
+        self.update: Optional[Update] = None
         self.candidates: Optional[List[EFBChannelChatIDStr]] = None
 
     @property
@@ -71,11 +72,9 @@ class ChatListStorage:
             if i.module_id not in self.channels and i.module_id in coordinator.slaves:
                 self.channels[i.module_id] = coordinator.slaves[i.module_id]
 
-    def set_chat_suggestion(self, update: telegram.Update,
-                            candidates: List[EFBChannelChatIDStr]):
-        """Set suggested chats of a message without recipient indicated."""
+    def set_chat_suggestion(self, update: Update):
+        """Set suggestion message without recipient indicated."""
         self.update = update
-        self.candidates = candidates
 
 
 class ChatBindingManager(LocaleMixin):
@@ -222,22 +221,25 @@ class ChatBindingManager(LocaleMixin):
             links = self.db.get_chat_assoc(
                 master_uid=utils.chat_id_to_str(self.channel.channel_id, message.chat.id))
             if links:
-                return self.link_chat_gen_list(message.chat.id, pattern=" ".join(args), chats=links)
+                return self.link_chat_gen_list(message.chat.id, pattern=" ".join(args),
+                                               chats=links, filter_availability=False)
         elif message.forward_from_chat and \
                 message.forward_from_chat.type == telegram.Chat.CHANNEL:
             chat_id = message.forward_from_chat.id
             links = self.db.get_chat_assoc(
                 master_uid=utils.chat_id_to_str(self.channel.channel_id, chat_id))
             if links:
-                return self.link_chat_gen_list(message.chat.id, pattern=" ".join(args), chats=links)
+                return self.link_chat_gen_list(message.chat.id, pattern=" ".join(args),
+                                               chats=links, filter_availability=False)
 
         return self.link_chat_gen_list(message.from_user.id, pattern=" ".join(args))
 
     def slave_chats_pagination(self, storage_id: Tuple[TelegramChatID, TelegramMessageID],
                                offset: int = 0,
                                pattern: Optional[str] = "",
-                               source_chats: Optional[List[EFBChannelChatIDStr]] = None) \
-            -> Tuple[List[str], List[List[telegram.InlineKeyboardButton]]]:
+                               source_chats: Optional[List[EFBChannelChatIDStr]] = None,
+                               filter_availability: bool = True) \
+            -> Tuple[List[str], List[List[InlineKeyboardButton]]]:
         """
         Generate a list of (list of) `InlineKeyboardButton`s of chats in slave channels,
         based on the status of message located by `storage_id` and the paging from
@@ -249,6 +251,8 @@ class ChatBindingManager(LocaleMixin):
             offset (int): Offset for pagination
             source_chats (Optional[List[str]]): A list of chats used to generate the pagination list.
                 Each str is in the format of "{channel_id}.{chat_uid}".
+            filter_availability (bool): Whether to filter chats based on the availabilities.
+                Only works when ``source_chats`` is specified.
 
         Returns:
             Tuple[List[str], List[List[telegram.InlineKeyboardButton]]]:
@@ -287,7 +291,7 @@ class ChatBindingManager(LocaleMixin):
                     channel_id, chat_uid, _ = utils.chat_id_str_to_id(s_chat)
                     with suppress(NameError):
                         coordinator.get_module_by_id(channel_id)
-                    chat = self.chat_manager.get_chat(channel_id, chat_uid)
+                    chat = self.chat_manager.get_chat(channel_id, chat_uid, build_dummy=not filter_availability)
                     if not chat:
                         self.logger.debug("slave_chats_pagination with chat list: Chat %s not found.", s_chat)
                         continue
@@ -307,7 +311,7 @@ class ChatBindingManager(LocaleMixin):
             legend.append(f"{ch.channel_emoji}: {ch.channel_name}")
 
         # Build inline button list
-        chat_btn_list: List[List[telegram.InlineKeyboardButton]] = []
+        chat_btn_list: List[List[InlineKeyboardButton]] = []
         chats_per_page = self.channel.flag("chats_per_page")
         for idx in range(offset, min(offset + chats_per_page, chat_list.length)):
             chat = chat_list.chats[idx]
@@ -319,26 +323,27 @@ class ChatBindingManager(LocaleMixin):
             chat_name = chat.long_name
             button_text = f"{chat.channel_emoji}{chat_type}{mode}: {chat_name}"
             button_callback = f"chat {idx}"
-            chat_btn_list.append([telegram.InlineKeyboardButton(button_text, callback_data=button_callback)])
+            chat_btn_list.append([InlineKeyboardButton(button_text, callback_data=button_callback)])
 
         # Pagination
-        page_number_row: List[telegram.InlineKeyboardButton] = []
+        page_number_row: List[InlineKeyboardButton] = []
 
         if offset - chats_per_page >= 0:
-            page_number_row.append(telegram.InlineKeyboardButton(self._("< Prev"),
-                                                                 callback_data=f"offset {offset - chats_per_page}"))
-        page_number_row.append(telegram.InlineKeyboardButton(self._("Cancel"),
-                                                             callback_data=Flags.CANCEL_PROCESS))
+            page_number_row.append(InlineKeyboardButton(self._("< Prev"),
+                                                        callback_data=f"offset {offset - chats_per_page}"))
+        page_number_row.append(InlineKeyboardButton(self._("Cancel"),
+                                                    callback_data=Flags.CANCEL_PROCESS))
         if offset + chats_per_page < chat_list.length:
-            page_number_row.append(telegram.InlineKeyboardButton(self._("Next >"),
-                                                                 callback_data=f"offset {offset + chats_per_page}"))
+            page_number_row.append(InlineKeyboardButton(self._("Next >"),
+                                                        callback_data=f"offset {offset + chats_per_page}"))
         chat_btn_list.append(page_number_row)
 
         return legend, chat_btn_list
 
     def link_chat_gen_list(self, chat_id: TelegramChatID,
                            message_id: TelegramMessageID = None, offset: int = 0,
-                           pattern: str = "", chats: List[EFBChannelChatIDStr] = None):
+                           pattern: str = "", chats: List[EFBChannelChatIDStr] = None,
+                           filter_availability: bool = True):
         """
         Generate the list for chat linking, and update it to a message.
 
@@ -348,6 +353,8 @@ class ChatBindingManager(LocaleMixin):
             offset: Offset for pagination.
             pattern (str): Regex expression to filter chats.
             chats (List[str]): Specified chats to link
+            filter_availability (bool): Whether to show only chats that are available.
+                Only works when ``chats`` are specified.
 
         Returns:
             int: The next state
@@ -355,7 +362,7 @@ class ChatBindingManager(LocaleMixin):
 
         if message_id is None:
             message_id = self.bot.send_message(chat_id, self._("Processing...")).message_id
-        self.bot.send_chat_action(chat_id, telegram.ChatAction.TYPING)
+        self.bot.send_chat_action(chat_id, ChatAction.TYPING)
         if chats:
             msg_text = self._("This Telegram group is currently linked with...")
         else:
@@ -365,12 +372,13 @@ class ChatBindingManager(LocaleMixin):
         legend, chat_btn_list = self.slave_chats_pagination((chat_id, message_id),
                                                             offset,
                                                             pattern=pattern,
-                                                            source_chats=chats)
+                                                            source_chats=chats,
+                                                            filter_availability=filter_availability)
         for i in legend:
             msg_text += "%s\n" % i
 
         self.bot.edit_message_text(chat_id=chat_id, message_id=message_id, text=msg_text,
-                                   reply_markup=telegram.InlineKeyboardMarkup(chat_btn_list))
+                                   reply_markup=InlineKeyboardMarkup(chat_btn_list))
 
         self.link_handler.conversations[(chat_id, message_id)] = Flags.LINK_CONFIRM
 
@@ -435,20 +443,20 @@ class ChatBindingManager(LocaleMixin):
                    f"startgroup={urllib.parse.quote(utils.b64en(utils.message_id_to_str(tg_chat_id, tg_msg_id)))}"
         self.logger.debug("Telegram start trigger for linking chat: %s", link_url)
         if chat.linked:
-            btn_list = [telegram.InlineKeyboardButton(self._("Relink"), url=link_url),
-                        telegram.InlineKeyboardButton(self._("Restore"), callback_data="unlink 0")]
+            btn_list = [InlineKeyboardButton(self._("Relink"), url=link_url),
+                        InlineKeyboardButton(self._("Restore"), callback_data="unlink 0")]
         else:
-            btn_list = [telegram.InlineKeyboardButton(self._("Link"), url=link_url)]
-        btn_list.append(telegram.InlineKeyboardButton(self._("Manual {link_or_relink}")
-                                                      .format(link_or_relink=btn_list[0].text),
-                                                      callback_data="manual_link 0"))
+            btn_list = [InlineKeyboardButton(self._("Link"), url=link_url)]
+        btn_list.append(InlineKeyboardButton(self._("Manual {link_or_relink}")
+                                             .format(link_or_relink=btn_list[0].text),
+                                             callback_data="manual_link 0"))
         buttons = [btn_list,
-                   [telegram.InlineKeyboardButton(self._("Cancel"), callback_data=Flags.CANCEL_PROCESS)]]
+                   [InlineKeyboardButton(self._("Cancel"), callback_data=Flags.CANCEL_PROCESS)]]
 
         self.bot.edit_message_text(text=txt,
                                    chat_id=tg_chat_id,
                                    message_id=tg_msg_id,
-                                   reply_markup=telegram.InlineKeyboardMarkup(buttons),
+                                   reply_markup=InlineKeyboardMarkup(buttons),
                                    parse_mode='HTML')
 
     def link_chat_exec(self, update: Update, context: CallbackContext) -> int:
@@ -487,9 +495,9 @@ class ChatBindingManager(LocaleMixin):
                 .format(chat_display_name=html.escape(chat_display_name),
                         code=html.escape(utils.b64en(utils.message_id_to_str(tg_chat_id, tg_msg_id))))
             self.bot.edit_message_text(text=txt, chat_id=tg_chat_id, message_id=tg_msg_id,
-                                       reply_markup=telegram.InlineKeyboardMarkup(
-                                           [[telegram.InlineKeyboardButton(self._("Cancel"),
-                                                                           callback_data=Flags.CANCEL_PROCESS)]]),
+                                       reply_markup=InlineKeyboardMarkup(
+                                           [[InlineKeyboardButton(self._("Cancel"),
+                                                                  callback_data=Flags.CANCEL_PROCESS)]]),
                                        parse_mode='HTML')
             return Flags.LINK_EXEC
         else:
@@ -583,7 +591,7 @@ class ChatBindingManager(LocaleMixin):
                 return self.bot.send_message(update.message.chat.id,
                                              self._("Send `/unlink_all` to a group to unlink all remote chats "
                                                     "from it."),
-                                             parse_mode=telegram.ParseMode.MARKDOWN,
+                                             parse_mode=ParseMode.MARKDOWN,
                                              reply_to_message_id=update.message.message_id)
 
     def start_chat_list(self, update: Update, context: CallbackContext):
@@ -619,7 +627,7 @@ class ChatBindingManager(LocaleMixin):
         """
         if message_id is None:
             message_id = self.bot.send_message(chat_id, text=self._("Processing...")).message_id
-        self.bot.send_chat_action(chat_id, telegram.ChatAction.TYPING)
+        self.bot.send_chat_action(chat_id, ChatAction.TYPING)
 
         if chats and len(chats):
             if len(chats) == 1:
@@ -627,7 +635,7 @@ class ChatBindingManager(LocaleMixin):
                 # TODO: Channel might be gone, add a check here.
                 chat = self.chat_manager.get_chat(slave_channel_id, slave_chat_id)
                 if chat:
-                    msg_text = self._('This group is linked to {0}'
+                    msg_text = self._('This group is linked to {0}. '
                                       'Send a message to this group to deliver it to the chat.\n'
                                       'Do NOT reply to this system message.').format(chat.full_name)
 
@@ -669,7 +677,7 @@ class ChatBindingManager(LocaleMixin):
         self.bot.edit_message_text(text=msg_text,
                                    chat_id=chat_id,
                                    message_id=message_id,
-                                   reply_markup=telegram.InlineKeyboardMarkup(chat_btn_list))
+                                   reply_markup=InlineKeyboardMarkup(chat_btn_list))
 
         self.chat_head_handler.conversations[(chat_id, message_id)] = Flags.CHAT_HEAD_CONFIRM
 
@@ -725,7 +733,7 @@ class ChatBindingManager(LocaleMixin):
         update.callback_query.answer()
         return ConversationHandler.END
 
-    def register_suggestions(self, update: telegram.Update,
+    def register_suggestions(self, update: Update,
                              candidates: List[EFBChannelChatIDStr],
                              chat_id: TelegramChatID, message_id: TelegramMessageID):
         storage_id = (chat_id, message_id)
@@ -736,12 +744,13 @@ class ChatBindingManager(LocaleMixin):
             # Remove message from cache and return
             del self.msg_storage[storage_id]
             return
-        self.msg_storage[storage_id].set_chat_suggestion(update, candidates)
+        # chat_list: Optional[ChatListStorage] = self.msg_storage.get(storage_id, None)
+        self.msg_storage[storage_id].set_chat_suggestion(update)
         self.bot.edit_message_text(text=self._("Error: No recipient specified.\n"
                                                "Please reply to a previous message, "
                                                "or choose a recipient:\n\nLegend:\n") + "\n".join(legends),
                                    chat_id=chat_id, message_id=message_id,
-                                   reply_markup=telegram.InlineKeyboardMarkup(buttons))
+                                   reply_markup=InlineKeyboardMarkup(buttons))
         self.suggestion_handler.conversations[storage_id] = Flags.SUGGEST_RECIPIENTS
 
     def suggested_recipient(self, update: Update, context: CallbackContext):
@@ -762,8 +771,8 @@ class ChatBindingManager(LocaleMixin):
                                            chat_id=chat_id,
                                            message_id=msg_id)
             update = self.msg_storage[storage_id].update
-            candidates = self.msg_storage[storage_id].candidates
-            if candidates is None:
+            chats = self.msg_storage[storage_id].chats
+            if not chats:
                 self.bot.edit_message_text(text=self._("Error: No recipient specified.\n"
                                                        "Please reply to a previous message.\n\n"
                                                        "Session expired, please try again."),
@@ -771,19 +780,12 @@ class ChatBindingManager(LocaleMixin):
                                            message_id=msg_id)
                 update.callback_query.answer()
                 return ConversationHandler.END
-            slave_chat_id = candidates[int(param.split(' ', 1)[1])]
-            module_id, chat_uid, _ = utils.chat_id_str_to_id(slave_chat_id)
-            chat = self.chat_manager.get_chat(module_id, chat_uid)
+            slave_chat = chats[int(param.split(' ', 1)[1])]
+            slave_chat_id = utils.chat_id_to_str(chat=slave_chat)
             self.channel.master_messages.process_telegram_message(update, context, slave_chat_id)
-            if chat:
-                self.bot.edit_message_text(text=self._("Delivering the message to {0}.").format(chat.full_name),
-                                           chat_id=chat_id,
-                                           message_id=msg_id)
-            else:
-                self.bot.edit_message_text(text=self._("Delivering the message to {module_id} {chat_uid}.")
-                                           .format(module_id=module_id, chat_uid=chat_uid),
-                                           chat_id=chat_id,
-                                           message_id=msg_id)
+            self.bot.edit_message_text(text=self._("Delivering the message to {0}.").format(slave_chat.full_name),
+                                       chat_id=chat_id,
+                                       message_id=msg_id)
         elif param == Flags.CANCEL_PROCESS:
             self.bot.edit_message_text(text=self._("Error: No recipient specified.\n"
                                                    "Please reply to a previous message."),
@@ -883,7 +885,7 @@ class ChatBindingManager(LocaleMixin):
                                                        "({channel_name}, {channel_id}).")
                                         .format(channel_name=channel.channel_name, channel_id=channel_id,
                                                 chat_uid=chat_uid))
-        except telegram.TelegramError as e:
+        except TelegramError as e:
             self.logger.exception("Error occurred while update chat details.")
             return self.bot.reply_error(update, self._('Error occurred while update chat details.\n'
                                                        '{0}'.format(e.message)))
