@@ -7,7 +7,6 @@ from PIL import Image, WebPImagePlugin
 from bullet import YesNo, Numbers, Bullet
 from pkg_resources import resource_filename
 
-import bullet
 import cjkwrap
 from ruamel.yaml import YAML
 from telegram import Bot, TelegramError
@@ -250,8 +249,8 @@ def setup_telegram_bot(data):
         prompt_yes = _("Yes, please tell me how to make one.")
         prompt_no = _("No, I have already made one according to the docs.")
 
-        choices = bullet.Bullet(prompt=_("Do you need help creating a bot?"),
-                                choices=[prompt_no, prompt_yes])
+        choices = Bullet(prompt=_("Do you need help creating a bot?"),
+                         choices=[prompt_no, prompt_yes])
         answer = choices.launch()
 
         if answer == prompt_yes:
@@ -286,24 +285,7 @@ def setup_telegram_bot(data):
                 "\n"
                 "Send /setprivacy to BotFather, choose the bot you just "
                 "created, then choose “Disable”. This will allow your bot to "
-                "process all messages in groups it joined, not just commands.\n"
-                "\n"
-                "Send /setcommands to BotFather, choose the bot you just "
-                "created, then copy and paste the following few lines to "
-                "BotFather. This will allow your bot to give you a list of "
-                "commands when you need them."
-            ))
-            print()
-            print(_(
-                "help - Show commands list.\n"
-                "link - Link a remote chat to a group.\n"
-                "unlink_all - Unlink all remote chats from a group.\n"
-                "info - Display information of the current Telegram chat.\n"
-                "chat - Generate a chat head.\n"
-                "extra - Access additional features from Slave Channels.\n"
-                "update_info - Update info of linked Telegram group.\n"
-                "react - Send a reaction to a message, or show a list of reactors.\n"
-                "rm - Remove a message from its remote chat."
+                "process all messages in groups it joined, not just commands."
             ))
             print()
             input(_("Press ENTER/RETURN to continue..."))
@@ -311,6 +293,35 @@ def setup_telegram_bot(data):
         data.data['token'] = input_bot_token(data)
 
 
+def setup_telegram_bot_commands_list(data):
+    prompt_yes = _("Yes, please update.")
+    prompt_no = _("No, I want to keep the old commands list.")
+
+    choices = Bullet(prompt=_("Do you want to update the list of commands of your bot?"),
+                     choices=[prompt_yes, prompt_no])
+    answer = choices.launch()
+
+    if answer == prompt_yes:
+        print(_("Updating commands list..."), end="", flush=True)
+        Bot(data.data['token'], request=data.request).set_my_commands(
+            [
+                ("help", _("Show commands list.")),
+                ("link", _("Link a remote chat to a group.")),
+                ("unlink_all", _("Unlink all remote chats from a group.")),
+                ("info", _("Display information of the current Telegram chat.")),
+                ("chat", _("Generate a chat head.")),
+                ("extra", _("Access additional features from Slave Channels.")),
+                ("update_info", _("Update info of linked Telegram group.")),
+                ("react", _("Send a reaction to a message, or show a list of reactors.")),
+                ("rm", _("Remove a message from its remote chat.")),
+            ]
+        )
+
+        print(_("OK"))
+        print()
+        input(_("Press ENTER/RETURN to continue..."))
+
+
 def input_admin_ids(default=None):
     prompt = _("List of Admin User IDs, separated with comma: ")
     if default:
@@ -349,8 +360,8 @@ def setup_admins(data):
         prompt_yes = _("Yes, I want to know how to get my ID.")
         prompt_no = _("No, I already know my ID.")
 
-        choices = bullet.Bullet(prompt=_("Do you need help getting your ID?"),
-                                choices=[prompt_no, prompt_yes])
+        choices = Bullet(prompt=_("Do you need help getting your ID?"),
+                         choices=[prompt_no, prompt_yes])
         answer = choices.launch()
 
         if answer == prompt_yes:
@@ -454,6 +465,15 @@ flags_settings = {
          _('Enable experimental support to animated stickers. Note: you might '
            'need to install binary dependency "libcairo" to enable this '
            'feature.')
+         ),
+    "send_to_last_chat":
+        ("warn", 'choices', ["enabled", "warn", "disabled"],
+         _('Enable quick reply in non-linked chats.\n'
+           '\n'
+           '- enabled: Enable this feature without warning.\n'
+           '- warn: Enable this feature and issue warnings every time when you '
+           'switch a recipient with quick reply.\n'
+           '- disabled: Disable this feature.')
          )
 }
 
@@ -484,10 +504,15 @@ def setup_experimental_flags(data):
             print()
             print(key)
             print_wrapped(desc)
-            ans = Numbers(prompt=f"{key} [{default}]? ") \
+            ans = Numbers(prompt=f"{key} [{default}]? ", type=int) \
                 .launch(default=default)
             data.data['flags'][key] = ans
         elif cat == 'choices':
+            try:
+                assert isinstance(params, list)
+                default = params.index(default)
+            except ValueError:
+                default = 0
             print()
             print(key)
             print_wrapped(desc)
@@ -572,7 +597,7 @@ def prerequisites_check():
 
     print(_("Checking libwebp installation..."), end="", flush=True)
     Image.init()
-    if 'WEBP' not in Image.ID or not WebPImagePlugin.SUPPORTED:
+    if 'WEBP' not in Image.ID or not getattr(WebPImagePlugin, "SUPPORTED", None):
         print(_("FAILED"))
         print_wrapped(_("libwebp plugin is not detected by Pillow."))
         exit(1)
@@ -597,6 +622,7 @@ def wizard(profile, instance_id):
     print()
     setup_proxy(data)
     setup_telegram_bot(data)
+    setup_telegram_bot_commands_list(data)
     setup_admins(data)
     setup_experimental_flags(data)
     setup_network_configurations(data)
